@@ -21,6 +21,16 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 
 /* To know about __NR_socketcall */
 #include <asm/unistd.h>
+#ifdef CONFIG_COMPAT
+#include <linux/compat.h>
+#endif
+
+#ifdef __NR_socketcall
+	#define _HAS_SOCKETCALL
+#endif
+#if defined(CONFIG_X86_64) && defined(CONFIG_IA32_EMULATION)
+	#define _HAS_SOCKETCALL
+#endif
 
 /*
  * Various crap that a callback might need
@@ -30,6 +40,7 @@ struct event_filler_arguments {
 	char *buffer; /* the buffer that will be filled with the data */
 	u32 buffer_size; /* the space in the ring buffer available for this event */
 	u32 syscall_id; /* the system call ID */
+	const enum ppm_syscall_code *cur_g_syscall_code_routing_table;
 #ifdef PPM_ENABLE_SENTINEL
 	u32 sentinel;
 #endif
@@ -43,9 +54,10 @@ struct event_filler_arguments {
 	struct task_struct *sched_prev; /* for context switch events, the task that is being schduled out */
 	struct task_struct *sched_next; /* for context switch events, the task that is being schduled in */
 	char *str_storage; /* String storage. Size is one page. */
-#ifdef __NR_socketcall
 	unsigned long socketcall_args[6];
-#endif
+	bool is_socketcall;
+	int socketcall_syscall;
+	bool compat;
 	int fd; /* Passed by some of the fillers to val_to_ring to compute the snaplen dynamically */
 	bool enforce_snaplen;
 	int signo; /* Signal number */
@@ -123,6 +135,10 @@ u16 fd_to_socktuple(int fd, struct sockaddr *usrsockaddr, int ulen, bool use_use
 int addr_to_kernel(void __user *uaddr, int ulen, struct sockaddr *kaddr);
 int32_t parse_readv_writev_bufs(struct event_filler_arguments *args, const struct iovec __user *iovsrc, unsigned long iovcnt, int64_t retval, int flags);
 
+#ifdef CONFIG_COMPAT
+int32_t compat_parse_readv_writev_bufs(struct event_filler_arguments *args, const struct compat_iovec __user *iovsrc, unsigned long iovcnt, int64_t retval, int flags);
+#endif
+
 static inline int add_sentinel(struct event_filler_arguments *args)
 {
 #ifdef PPM_ENABLE_SENTINEL
@@ -131,9 +147,8 @@ static inline int add_sentinel(struct event_filler_arguments *args)
 		args->arg_data_offset += 4;
 		args->arg_data_size -= 4;
 		return PPM_SUCCESS;
-	} else {
-		return PPM_FAILURE_BUFFER_FULL;
 	}
+	return PPM_FAILURE_BUFFER_FULL;
 #else
 	return PPM_SUCCESS;
 #endif
