@@ -128,7 +128,7 @@ repos = {
 			"subdirs" : [
 				""
 			],
-			"page_pattern" : "/html/body//a[regex:test(@href, '^[5-9][0-9][0-9]')]/@href"
+			"page_pattern" : "/html/body//a[regex:test(@href, '^[5-9][0-9][0-9]|current')]/@href"
 		},
 
 		{
@@ -137,7 +137,7 @@ repos = {
 			"subdirs" : [
 				""
 			],
-			"page_pattern" : "/html/body//a[regex:test(@href, '^[5-9][0-9][0-9]')]/@href"
+			"page_pattern" : "/html/body//a[regex:test(@href, '^[5-9][0-9][0-9]|current')]/@href"
 		},
 
 		{
@@ -146,9 +146,33 @@ repos = {
 			"subdirs" : [
 				""
 			],
-			"page_pattern" : "/html/body//a[regex:test(@href, '^[4-9][0-9][0-9]')]/@href"
+			"page_pattern" : "/html/body//a[regex:test(@href, '^[4-9][0-9][0-9]|current')]/@href"
 		}
-	]
+	],
+
+	"Debian": [
+        {
+            "root": "https://mirrors.kernel.org/debian/pool/main/l/",
+            "discovery_pattern": "/html/body/pre/a[@href = 'linux/']/@href",
+            "subdirs": [""],
+            "page_pattern": "/html/body//a[regex:test(@href, '^linux-(image|headers)-[3-9]\.[0-9]+\.[0-9]+.*amd64.deb$')]/@href",
+            "exclude_patterns": ["-rt", "dbg", "trunk", "all"]
+        },
+        {
+            "root": "http://security.debian.org/pool/updates/main/l/",
+            "discovery_pattern": "/html/body/table//tr/td/a[@href = 'linux/']/@href",
+            "subdirs": [""],
+            "page_pattern": "/html/body//a[regex:test(@href, '^linux-(image|headers)-[3-9]\.[0-9]\.[0-9]+.*amd64.deb$')]/@href",
+            "exclude_patterns": ["-rt", "dbg", "trunk", "all"]
+        },
+        {
+            "root": "http://mirrors.kernel.org/debian/pool/main/l/",
+            "discovery_pattern": "/html/body/pre/a[@href = 'linux-tools/']/@href",
+            "subdirs": [""],
+            "page_pattern": "/html/body//a[regex:test(@href, '^linux-kbuild-.*amd64.deb$')]/@href",
+            "exclude_patterns": ["-rt", "dbg", "trunk", "all"]
+        }
+    ]
 }
 
 #
@@ -157,7 +181,7 @@ repos = {
 # links will be found automagically without needing to write any single line of
 # code.
 #
-packages = {}
+urls = set()
 
 if len(sys.argv) < 2 or not sys.argv[1] in repos:
 	sys.stderr.write("Usage: " + sys.argv[0] + " <distro>\n")
@@ -184,13 +208,15 @@ for repo in repos[sys.argv[1]]:
 				rpms = html.fromstring(page).xpath(repo["page_pattern"], namespaces = {"regex": "http://exslt.org/regular-expressions"})
 
 				for rpm in rpms:
-					if not rpm in packages:
-						packages[rpm] = source + rpm
+					if "exclude_patterns" in repo and any(x in rpm for x in repo["exclude_patterns"]):
+						continue
+					else:
+						urls.add(source + str(urllib2.unquote(rpm)))
 			except:
 				continue
 
 #
 # Print URLs to stdout
 #
-for rpm, url in packages.iteritems():
+for url in urls:
 	print(url)
\ No newline at end of file
