@@ -214,6 +214,8 @@ typedef struct scap_threadinfo
 	int64_t vpid;
 	char cgroups[SCAP_MAX_CGROUPS_SIZE];
 	uint16_t cgroups_len;
+	char root[SCAP_MAX_PATH_SIZE];
+	int filtered_out; ///< nonzero if this entry should not be saved to file
 	scap_fdinfo* fdlist; ///< The fd table for this process
 	UT_hash_handle hh; ///< makes this structure hashable
 }scap_threadinfo;
@@ -853,6 +855,9 @@ int32_t scap_enable_dynamic_snaplen(scap_t* handle);
 int32_t scap_disable_dynamic_snaplen(scap_t* handle);
 void scap_proc_free_table(scap_t* handle);
 void scap_refresh_iflist(scap_t* handle);
+void scap_set_refresh_proc_table_when_saving(scap_t* handle, bool refresh);
+uint64_t scap_ftell(scap_t *handle);
+void scap_fseek(scap_t *handle, uint64_t off);
 
 #ifdef __cplusplus
 }
