@@ -72,6 +72,7 @@ int32_t scap_proc_fill_info_from_stats(char* procdirname, struct scap_threadinfo
 	tinfo->vmswap_kb = 0;
 	tinfo->pfmajor = 0;
 	tinfo->pfminor = 0;
+	tinfo->filtered_out = 0;
 
 	snprintf(filename, sizeof(filename), "%sstatus", procdirname);
 
@@ -406,6 +407,20 @@ int32_t scap_getpid_global(scap_t* handle, int64_t* pid)
 #endif	
 }
 
+int32_t scap_proc_fill_root(struct scap_threadinfo* tinfo, const char* procdirname)
+{
+	char root_path[SCAP_MAX_PATH_SIZE];
+	snprintf(root_path, sizeof(root_path), "%sroot", procdirname);
+	if ( readlink(root_path, tinfo->root, sizeof(tinfo->root)) > 0)
+	{
+		return SCAP_SUCCESS;
+	}
+	else
+	{
+		return SCAP_FAILURE;
+	}
+}
+
 //
 // Add a process to the list by parsing its entry under /proc
 //
@@ -653,6 +668,16 @@ static int32_t scap_proc_add_from_proc(scap_t* handle, uint32_t tid, int parentt
 		tinfo->vpid = tinfo->pid;
 	}
 
+	//
+	// set the current root of the process
+	//
+	if(SCAP_FAILURE == scap_proc_fill_root(tinfo, dir_name))
+	{
+		snprintf(error, SCAP_LASTERR_SIZE, "can't fill root for %s", dir_name);
+		free(tinfo);
+		return SCAP_FAILURE;
+	}
+
 	//
 	// if tid_to_scan is set we assume this is a runtime lookup so no
 	// need to use the table
