@@ -69,7 +69,10 @@ static int32_t scap_write_proc_fds(scap_t *handle, struct scap_threadinfo *tinfo
 	//
 	HASH_ITER(hh, tinfo->fdlist, fdi, tfdi)
 	{
-		totlen += scap_fd_info_len(fdi);
+		if(fdi->type != SCAP_FD_UNINITIALIZED)
+		{
+			totlen += scap_fd_info_len(fdi);
+		}
 	}
 
 	//
@@ -98,9 +101,12 @@ static int32_t scap_write_proc_fds(scap_t *handle, struct scap_threadinfo *tinfo
 	//
 	HASH_ITER(hh, tinfo->fdlist, fdi, tfdi)
 	{
-		if(scap_fd_write_to_disk(handle, fdi, f) != SCAP_SUCCESS)
+		if(fdi->type != SCAP_FD_UNINITIALIZED)
 		{
-			return SCAP_FAILURE;
+			if(scap_fd_write_to_disk(handle, fdi, f) != SCAP_SUCCESS)
+			{
+				return SCAP_FAILURE;
+			}
 		}
 	}
 
@@ -137,10 +143,13 @@ static int32_t scap_write_fdlist(scap_t *handle, gzFile f)
 
 	HASH_ITER(hh, handle->m_proclist, tinfo, ttinfo)
 	{
-		res = scap_write_proc_fds(handle, tinfo, f);
-		if(res != SCAP_SUCCESS)
+		if(!tinfo->filtered_out)
 		{
-			return res;
+			res = scap_write_proc_fds(handle, tinfo, f);
+			if(res != SCAP_SUCCESS)
+			{
+				return res;
+			}
 		}
 	}
 
@@ -161,39 +170,44 @@ static int32_t scap_write_proclist(scap_t *handle, gzFile f)
 	uint16_t exelen;
 	uint16_t argslen;
 	uint16_t cwdlen;
+	uint16_t rootlen;
 
 	//
 	// First pass pass of the table to calculate the length
 	//
 	HASH_ITER(hh, handle->m_proclist, tinfo, ttinfo)
 	{
-		totlen += (uint32_t)
-		    (sizeof(uint64_t) +	// tid
-		    sizeof(uint64_t) +	// pid
-		    sizeof(uint64_t) +	// ptid
-		    2 + strnlen(tinfo->comm, SCAP_MAX_PATH_SIZE) +
-		    2 + strnlen(tinfo->exe, SCAP_MAX_PATH_SIZE) +
-		    2 + tinfo->args_len +
-		    2 + strnlen(tinfo->cwd, SCAP_MAX_PATH_SIZE) +
-		    sizeof(uint64_t) +	// fdlimit
-		    sizeof(uint32_t) +	// uid
-		    sizeof(uint32_t) +	// gid
-		    sizeof(uint32_t) +  // vmsize_kb
-		    sizeof(uint32_t) +  // vmrss_kb
-		    sizeof(uint32_t) +  // vmswap_kb
-		    sizeof(uint64_t) +  // pfmajor
-		    sizeof(uint64_t) +  // pfminor
-		    2 + tinfo->env_len +
-		    sizeof(int64_t) +  // vtid
-		    sizeof(int64_t) +  // vpid
-		    2 + tinfo->cgroups_len +
-		    sizeof(uint32_t));
+		if(!tinfo->filtered_out)
+		{
+			totlen += (uint32_t)
+				(sizeof(uint64_t) +	// tid
+				sizeof(uint64_t) +	// pid
+				sizeof(uint64_t) +	// ptid
+				2 + strnlen(tinfo->comm, SCAP_MAX_PATH_SIZE) +
+				2 + strnlen(tinfo->exe, SCAP_MAX_PATH_SIZE) +
+				2 + tinfo->args_len +
+				2 + strnlen(tinfo->cwd, SCAP_MAX_PATH_SIZE) +
+				sizeof(uint64_t) +	// fdlimit
+				sizeof(uint32_t) +	// uid
+				sizeof(uint32_t) +	// gid
+				sizeof(uint32_t) +  // vmsize_kb
+				sizeof(uint32_t) +  // vmrss_kb
+				sizeof(uint32_t) +  // vmswap_kb
+				sizeof(uint64_t) +  // pfmajor
+				sizeof(uint64_t) +  // pfminor
+				2 + tinfo->env_len +
+				sizeof(int64_t) +  // vtid
+				sizeof(int64_t) +  // vpid
+				2 + tinfo->cgroups_len +
+				sizeof(uint32_t) +
+				2 + strnlen(tinfo->root, SCAP_MAX_PATH_SIZE));
+		}
 	}
 
 	//
 	// Create the block
 	//
-	bh.block_type = PL_BLOCK_TYPE_V4;
+	bh.block_type = PL_BLOCK_TYPE_V5;
 	bh.block_total_length = scap_normalize_block_len(sizeof(block_header) + totlen + 4);
 
 	if(gzwrite(f, &bh, sizeof(bh)) != sizeof(bh))
@@ -207,10 +221,16 @@ static int32_t scap_write_proclist(scap_t *handle, gzFile f)
 	//
 	HASH_ITER(hh, handle->m_proclist, tinfo, ttinfo)
 	{
+		if(tinfo->filtered_out)
+		{
+			continue;
+		}
+
 		commlen = (uint16_t)strnlen(tinfo->comm, SCAP_MAX_PATH_SIZE);
 		exelen = (uint16_t)strnlen(tinfo->exe, SCAP_MAX_PATH_SIZE);
 		argslen = tinfo->args_len;
 		cwdlen = (uint16_t)strnlen(tinfo->cwd, SCAP_MAX_PATH_SIZE);
+		rootlen = (uint16_t)strnlen(tinfo->root, SCAP_MAX_PATH_SIZE);
 
 		if(gzwrite(f, &(tinfo->tid), sizeof(uint64_t)) != sizeof(uint64_t) ||
 		        gzwrite(f, &(tinfo->pid), sizeof(uint64_t)) != sizeof(uint64_t) ||
@@ -237,7 +257,9 @@ static int32_t scap_write_proclist(scap_t *handle, gzFile f)
 		        gzwrite(f, &(tinfo->vtid), sizeof(int64_t)) != sizeof(int64_t) ||
 		        gzwrite(f, &(tinfo->vpid), sizeof(int64_t)) != sizeof(int64_t) ||
 		        gzwrite(f, &(tinfo->cgroups_len), sizeof(uint16_t)) != sizeof(uint16_t) ||
-		        gzwrite(f, tinfo->cgroups, tinfo->cgroups_len) != tinfo->cgroups_len)
+		        gzwrite(f, tinfo->cgroups, tinfo->cgroups_len) != tinfo->cgroups_len ||
+		        gzwrite(f, &rootlen, sizeof(uint16_t)) != sizeof(uint16_t) ||
+		        gzwrite(f, tinfo->root, rootlen) != rootlen)
 		{
 			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "error writing to file (2)");
 			return SCAP_FAILURE;
@@ -549,7 +571,7 @@ static scap_dumper_t *scap_setup_dump(scap_t *handle, gzFile f, const char *fnam
 	// between opening the handle and starting the dump
 	//
 #if defined(HAS_CAPTURE)
-	if(handle->m_file == NULL)
+	if(handle->m_file == NULL && handle->refresh_proc_table_when_saving)
 	{
 		proc_entry_callback tcb = handle->m_proc_callback;
 		handle->m_proc_callback = NULL;
@@ -602,7 +624,6 @@ static scap_dumper_t *scap_setup_dump(scap_t *handle, gzFile f, const char *fnam
 	//
 	// Write the fd lists
 	//
-
 	if(scap_write_fdlist(handle, f) != SCAP_SUCCESS)
 	{
 		return NULL;
@@ -818,6 +839,8 @@ static int32_t scap_read_proclist(scap_t *handle, gzFile f, uint32_t block_lengt
 	tinfo.vtid = -1;
 	tinfo.vpid = -1;
 	tinfo.cgroups_len = 0;
+	tinfo.filtered_out = 0;
+	tinfo.root[0] = 0;
 
 	while(((int32_t)block_length - (int32_t)totreadsize) >= 4)
 	{
@@ -976,6 +999,7 @@ static int32_t scap_read_proclist(scap_t *handle, gzFile f, uint32_t block_lengt
 		case PL_BLOCK_TYPE_V3:
 		case PL_BLOCK_TYPE_V3_INT:
 		case PL_BLOCK_TYPE_V4:
+		case PL_BLOCK_TYPE_V5:
 			//
 			// vmsize_kb
 			//
@@ -1018,7 +1042,8 @@ static int32_t scap_read_proclist(scap_t *handle, gzFile f, uint32_t block_lengt
 
 			if(block_type == PL_BLOCK_TYPE_V3 ||
 				block_type == PL_BLOCK_TYPE_V3_INT ||
-				block_type == PL_BLOCK_TYPE_V4)
+				block_type == PL_BLOCK_TYPE_V4 ||
+				block_type == PL_BLOCK_TYPE_V5)
 			{
 				//
 				// env
@@ -1044,7 +1069,7 @@ static int32_t scap_read_proclist(scap_t *handle, gzFile f, uint32_t block_lengt
 				totreadsize += readsize;
 			}
 
-			if(block_type == PL_BLOCK_TYPE_V4)
+			if(block_type == PL_BLOCK_TYPE_V4 || block_type == PL_BLOCK_TYPE_V5)
 			{
 				//
 				// vtid
@@ -1081,6 +1106,28 @@ static int32_t scap_read_proclist(scap_t *handle, gzFile f, uint32_t block_lengt
 				CHECK_READ_SIZE(readsize, stlen);
 
 				totreadsize += readsize;
+
+				if(block_type == PL_BLOCK_TYPE_V5)
+				{
+					readsize = gzread(f, &(stlen), sizeof(uint16_t));
+					CHECK_READ_SIZE(readsize, sizeof(uint16_t));
+
+					if(stlen > SCAP_MAX_PATH_SIZE)
+					{
+						snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "invalid rootlen %d", stlen);
+						return SCAP_FAILURE;
+					}
+
+					totreadsize += readsize;
+
+					readsize = gzread(f, tinfo.root, stlen);
+					CHECK_READ_SIZE(readsize, stlen);
+
+					// the string is not null-terminated on file
+					tinfo.root[stlen] = 0;
+
+					totreadsize += readsize;
+				}
 			}
 			break;
 		default:
@@ -1849,6 +1896,7 @@ int32_t scap_read_init(scap_t *handle, gzFile f)
 		case PL_BLOCK_TYPE_V2:
 		case PL_BLOCK_TYPE_V3:
 		case PL_BLOCK_TYPE_V4:
+		case PL_BLOCK_TYPE_V5:
 		case PL_BLOCK_TYPE_V1_INT:
 		case PL_BLOCK_TYPE_V2_INT:
 		case PL_BLOCK_TYPE_V3_INT:
@@ -1942,7 +1990,7 @@ int32_t scap_read_init(scap_t *handle, gzFile f)
 
 	if(!found_mi)
 	{
-		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "corrupted input file. Can't find machine info block.");			
+		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "corrupted input file. Can't find machine info block.");
 		return SCAP_FAILURE;
 	}
 
@@ -1958,18 +2006,6 @@ int32_t scap_read_init(scap_t *handle, gzFile f)
 		return SCAP_FAILURE;
 	}
 
-	if(!found_fdl)
-	{
-		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "corrupted input file. Can't find file descriptor list block.");			
-		return SCAP_FAILURE;
-	}
-
-	if(!found_pl)
-	{
-		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "corrupted input file. Can't find process list block.");			
-		return SCAP_FAILURE;
-	}
-
 	return SCAP_SUCCESS;
 }
 
@@ -1989,8 +2025,17 @@ int32_t scap_next_offline(scap_t *handle, OUT scap_evt **pevent, OUT uint16_t *p
 	// Read the block header
 	//
 	readsize = gzread(f, &bh, sizeof(bh));
+
 	if(readsize != sizeof(bh))
 	{
+		int err_no = 0;
+		const char* err_str = gzerror(f, &err_no);
+		if(err_no)
+		{
+			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "error reading file: %s, ernum=%d", err_str, err_no);
+			return SCAP_FAILURE;
+		}
+
 		if(readsize == 0)
 		{
 			//
@@ -2043,3 +2088,19 @@ int32_t scap_next_offline(scap_t *handle, OUT scap_evt **pevent, OUT uint16_t *p
 
 	return SCAP_SUCCESS;
 }
+
+uint64_t scap_ftell(scap_t *handle)
+{
+	gzFile f = handle->m_file;
+	ASSERT(f != NULL);
+
+	return gztell(f);
+}
+
+void scap_fseek(scap_t *handle, uint64_t off)
+{
+	gzFile f = handle->m_file;
+	ASSERT(f != NULL);
+
+	gzseek(f, off, SEEK_SET);
+}
