@@ -1044,6 +1044,10 @@ int lua_cbacks::get_container_table(lua_State *ls)
 		{
 			lua_pushstring(ls, "libvirt_lxc");
 		}
+		else if(it->second.m_type == CT_MESOS)
+		{
+			lua_pushstring(ls, "mesos");
+		}
 		else
 		{
 			ASSERT(false);
