@@ -23,7 +23,8 @@ enum sinsp_container_type
 	CT_DOCKER = 0,
 	CT_LXC = 1,
 	CT_LIBVIRT_LXC = 2,
-	CT_MESOS = 3
+	CT_MESOS = 3,
+	CT_RKT = 4
 };
 
 class sinsp_container_info
@@ -38,7 +39,6 @@ public:
 			m_container_port(0)
 		{
 		}
-
 		uint32_t m_host_ip;
 		uint16_t m_host_port;
 		uint16_t m_container_port;
@@ -56,6 +56,7 @@ public:
 	uint32_t m_container_ip;
 	vector<container_port_mapping> m_port_mappings;
 	map<string, string> m_labels;
+	string m_mesos_task_id;
 };
 
 class sinsp_container_manager
@@ -66,14 +67,18 @@ public:
 	const unordered_map<string, sinsp_container_info>* get_containers();
 	bool remove_inactive_containers();
 	void add_container(const sinsp_container_info& container_info);
-	bool get_container(const string& id, sinsp_container_info* container_info);
-	bool resolve_container_from_cgroups(const vector<pair<string, string>>& cgroups, bool query_os_for_missing_info, string* container_id);
+	bool get_container(const string& id, sinsp_container_info* container_info) const;
+	bool resolve_container(sinsp_threadinfo* tinfo, bool query_os_for_missing_info);
 	void dump_containers(scap_dumper_t* dumper);
 	string get_container_name(sinsp_threadinfo* tinfo);
+	bool set_mesos_task_id(sinsp_container_info* container, const string& task_id, int64_t ptid = -1);
+	string get_mesos_task_id(const string& container_id);
 
 private:
 	bool container_to_sinsp_event(const sinsp_container_info& container_info, sinsp_evt* evt, size_t evt_len);
 	bool parse_docker(sinsp_container_info* container);
+	bool parse_rkt(sinsp_container_info* container, const string& podid, const string& appname);
+	sinsp_container_info* get_container(const string& id);
 
 	sinsp* m_inspector;
 	unordered_map<string, sinsp_container_info> m_containers;
