@@ -177,6 +177,14 @@ public:
 		return m_info->flags;
 	}
 
+	/*!
+	\brief Get the event's category.
+	*/
+	inline ppm_event_category get_info_category()
+	{
+		return m_info->category;
+	}
+
 	/*!
 	  \brief Return the event direction: in or out.
 	*/
