@@ -35,6 +35,14 @@ template<> sinsp_fdinfo_t::sinsp_fdinfo()
 	m_usrstate = NULL;
 }
 
+template<> void sinsp_fdinfo_t::reset()
+{
+	m_type = SCAP_FD_UNINITIALIZED;
+	m_flags = FLAGS_NONE;
+	m_callbaks = NULL;
+	m_usrstate = NULL;
+}
+
 template<> string* sinsp_fdinfo_t::tostring()
 {
 	return &m_name;
@@ -130,38 +138,6 @@ template<> bool sinsp_fdinfo_t::set_net_role_by_guessing(sinsp* inspector,
 										  sinsp_fdinfo_t* pfdinfo,
 										  bool incoming)
 {
-/*
-	bool is_sip_local = 
-		inspector->get_ifaddr_list()->is_ipv4addr_in_local_machine(pfdinfo->m_sockinfo.m_ipv4info.m_fields.m_sip);
-	bool is_dip_local = 
-		inspector->get_ifaddr_list()->is_ipv4addr_in_local_machine(pfdinfo->m_sockinfo.m_ipv4info.m_fields.m_dip);
-
-	//
-	// If only the client is local, mark the role as client.
-	// If only the server is local, mark the role as server.
-	//
-	if(is_sip_local)
-	{
-		if(!is_dip_local)
-		{
-			pfdinfo->set_role_client();
-			return true;
-		}
-	}
-	else if(is_dip_local)
-	{
-		if(!is_sip_local)
-		{
-			pfdinfo->set_role_server();
-			return true;
-		}
-	}
-
-	//
-	// Both addresses are local
-	//
-	ASSERT(is_sip_local && is_dip_local);
-*/
 	//
 	// If this process owns the port, mark it as server, otherwise mark it as client
 	//
