@@ -46,7 +46,7 @@ public:
 	/*!
 	  \brief Constructs the filter.
 
-	  \param inspector Pointer to the inspector instance that will generate the 
+	  \param inspector Pointer to the inspector instance that will generate the
 	   events to be filtered.
 	  \param fltstr the filter string to compile.
 	  \param ttable_only for internal use only.
@@ -61,7 +61,7 @@ public:
 	  \brief Applies the filter to the given event.
 
 	  \param evt Pointer that needs to be filtered.
-	  \return true if the event is accepted by the filter, false if it's rejected. 
+	  \return true if the event is accepted by the filter, false if it's rejected.
 	*/
 	bool run(sinsp_evt *evt);
 
@@ -75,7 +75,7 @@ private:
 	char next();
 	bool compare_no_consume(const string& str);
 
-	vector<char> next_operand(bool expecting_first_operand);
+	vector<char> next_operand(bool expecting_first_operand, bool in_clause);
 	ppm_cmp_operator next_comparison_operator();
 	void parse_check(sinsp_filter_expression* parent_expr, boolop op);
 	void push_expression(boolop op);
