@@ -4,6 +4,8 @@
 
 #pragma once
 
+#ifdef HAS_CAPTURE
+
 #include "k8s_common.h"
 #include <map>
 #include <atomic>
@@ -49,4 +51,6 @@ private:
 inline void k8s_collector::stop()
 {
 	m_stopped = true;
-}
\ No newline at end of file
+}
+
+#endif // HAS_CAPTURE
\ No newline at end of file
