@@ -12,7 +12,7 @@
 #include <iostream>
 
 
-k8s_dispatcher::k8s_dispatcher(k8s_component::type t, k8s_state_s& state
+k8s_dispatcher::k8s_dispatcher(k8s_component::type t, k8s_state_t& state
 #ifndef K8S_DISABLE_THREAD
 	,std::mutex& mut
 #endif
@@ -87,7 +87,7 @@ bool k8s_dispatcher::is_ready(const std::string& msg)
 	return msg[msg.size() - 1] == '\n';
 }
 
-k8s_dispatcher::msg_data k8s_dispatcher::get_msg_data(const Json::Value& root)
+k8s_dispatcher::msg_data k8s_dispatcher::get_msg_data(Json::Value& root)
 {
 	msg_data data;
 	Json::Value evtype = root["type"];
@@ -107,6 +107,20 @@ k8s_dispatcher::msg_data k8s_dispatcher::get_msg_data(const Json::Value& root)
 		}
 	}
 	Json::Value object = root["object"];
+
+	// +++ for capture
+	Json::Value kind = object["kind"];
+	if(!kind.isNull() && kind.isString() && root["kind"].isNull())
+	{
+		root["kind"] = kind.asString();
+	}
+	Json::Value api_version = object["apiVersion"];
+	if(!api_version.isNull() && api_version.isString() && root["apiVersion"].isNull())
+	{
+		root["apiVersion"] = api_version.asString();
+	}
+	// --- for capture
+
 	if(!object.isNull() && object.isObject())
 	{
 		Json::Value meta = object["metadata"];
@@ -132,6 +146,24 @@ k8s_dispatcher::msg_data k8s_dispatcher::get_msg_data(const Json::Value& root)
 	return data;
 }
 
+void k8s_dispatcher::log_error(const Json::Value& root, const std::string& comp)
+{
+	std::string unk_err = "Unknown.";
+	std::ostringstream os;
+	os << "K8S server reported " << comp << " error: ";
+	if(!root.isNull())
+	{
+		Json::Value object = root["object"];
+		if(!object.isNull())
+		{
+			os << object.toStyledString();
+			unk_err.clear();
+		}
+	}
+	os << unk_err;
+	g_logger.log(os.str(), sinsp_logger::SEV_ERROR);
+}
+
 void k8s_dispatcher::handle_node(const Json::Value& root, const msg_data& data)
 {
 	K8S_LOCK_GUARD_MUTEX;
@@ -145,7 +177,7 @@ void k8s_dispatcher::handle_node(const Json::Value& root, const msg_data& data)
 			os << "ADDED message received for existing node [" << data.m_uid << "], updating only.";
 			g_logger.log(os.str(), sinsp_logger::SEV_INFO);
 		}
-		k8s_node_s& node = m_state.get_component<k8s_state_s::nodes, k8s_node_s>(m_state.get_nodes(), data.m_name, data.m_uid);
+		k8s_node_t& node = m_state.get_component<k8s_nodes, k8s_node_t>(m_state.get_nodes(), data.m_name, data.m_uid);
 		if(addresses.size() > 0)
 		{
 			node.set_host_ips(std::move(addresses));
@@ -174,7 +206,7 @@ void k8s_dispatcher::handle_node(const Json::Value& root, const msg_data& data)
 			g_logger.log(os.str(), sinsp_logger::SEV_ERROR);
 			return;
 		}
-		k8s_node_s& node = m_state.get_component<k8s_state_s::nodes, k8s_node_s>(m_state.get_nodes(), data.m_name, data.m_uid);
+		k8s_node_t& node = m_state.get_component<k8s_nodes, k8s_node_t>(m_state.get_nodes(), data.m_name, data.m_uid);
 		if(addresses.size() > 0)
 		{
 			node.add_host_ips(std::move(addresses));
@@ -202,7 +234,7 @@ void k8s_dispatcher::handle_node(const Json::Value& root, const msg_data& data)
 	}
 	else // COMPONENT_ERROR
 	{
-		g_logger.log("Bad NODE watch message.", sinsp_logger::SEV_ERROR);
+		log_error(root, "NODE");
 	}
 }
 
@@ -218,7 +250,7 @@ void k8s_dispatcher::handle_namespace(const Json::Value& root, const msg_data& d
 			os << "ADDED message received for existing namespace [" << data.m_uid << "], updating only.";
 			g_logger.log(os.str(), sinsp_logger::SEV_INFO);
 		}
-		k8s_ns_s& ns = m_state.get_component<k8s_state_s::namespaces, k8s_ns_s>(m_state.get_namespaces(), data.m_name, data.m_uid);
+		k8s_ns_t& ns = m_state.get_component<k8s_namespaces, k8s_ns_t>(m_state.get_namespaces(), data.m_name, data.m_uid);
 		Json::Value object = root["object"];
 		if(!object.isNull())
 		{
@@ -242,7 +274,7 @@ void k8s_dispatcher::handle_namespace(const Json::Value& root, const msg_data& d
 			g_logger.log(os.str(), sinsp_logger::SEV_ERROR);
 			return;
 		}
-		k8s_ns_s& ns = m_state.get_component<k8s_state_s::namespaces, k8s_ns_s>(m_state.get_namespaces(), data.m_name, data.m_uid);
+		k8s_ns_t& ns = m_state.get_component<k8s_namespaces, k8s_ns_t>(m_state.get_namespaces(), data.m_name, data.m_uid);
 		Json::Value object = root["object"];
 		if(!object.isNull())
 		{
@@ -266,7 +298,7 @@ void k8s_dispatcher::handle_namespace(const Json::Value& root, const msg_data& d
 	}
 	else // COMPONENT_ERROR
 	{
-		g_logger.log("Bad NAMESPACE watch message.", sinsp_logger::SEV_ERROR);
+		log_error(root, "NAMESPACE");
 	}
 }
 
@@ -285,16 +317,8 @@ void k8s_dispatcher::handle_pod(const Json::Value& root, const msg_data& data)
 				os << "ADDED message received for existing pod [" << data.m_uid << "], updating only.";
 				g_logger.log(os.str(), sinsp_logger::SEV_INFO);
 			}
-			k8s_pod_s& pod = m_state.get_component<k8s_state_s::pods, k8s_pod_s>(m_state.get_pods(), data.m_name, data.m_uid, data.m_namespace);
-			Json::Value metadata = object["metadata"];
-			if(!metadata.isNull())
-			{
-				k8s_pair_list entries = k8s_component::extract_object(metadata, "labels");
-				if(entries.size() > 0)
-				{
-					pod.set_labels(std::move(entries));
-				}
-			}
+			k8s_pod_t& pod = m_state.get_component<k8s_pods, k8s_pod_t>(m_state.get_pods(), data.m_name, data.m_uid, data.m_namespace);
+			handle_labels(pod, object["metadata"], "labels");
 			m_state.update_pod(pod, object, false);
 		}
 	}
@@ -310,22 +334,14 @@ void k8s_dispatcher::handle_pod(const Json::Value& root, const msg_data& data)
 				g_logger.log(os.str(), sinsp_logger::SEV_ERROR);
 				return;
 			}
-			k8s_pod_s& pod = m_state.get_component<k8s_state_s::pods, k8s_pod_s>(m_state.get_pods(), data.m_name, data.m_uid, data.m_namespace);
-			Json::Value metadata = object["metadata"];
-			if(!metadata.isNull())
-			{
-				k8s_pair_list entries = k8s_component::extract_object(metadata, "labels");
-				if(entries.size() > 0)
-				{
-					pod.add_labels(std::move(entries));
-				}
-			}
+			k8s_pod_t& pod = m_state.get_component<k8s_pods, k8s_pod_t>(m_state.get_pods(), data.m_name, data.m_uid, data.m_namespace);
+			handle_labels(pod, object["metadata"], "labels");
 			m_state.update_pod(pod, object, false);
 		}
 	}
 	else if(data.m_reason == COMPONENT_DELETED)
 	{
-		k8s_pod_s* pod = m_state.get_component<k8s_state_s::pods, k8s_pod_s>(m_state.get_pods(), data.m_uid);
+		k8s_pod_t* pod = m_state.get_component<k8s_pods, k8s_pod_t>(m_state.get_pods(), data.m_uid);
 		if(pod)
 		{
 			if(!m_state.delete_component(m_state.get_pods(), data.m_uid))
@@ -340,7 +356,7 @@ void k8s_dispatcher::handle_pod(const Json::Value& root, const msg_data& data)
 	}
 	else // COMPONENT_ERROR
 	{
-		g_logger.log("Bad POD watch message.", sinsp_logger::SEV_ERROR);
+		log_error(root, "POD");
 	}
 }
 
@@ -356,29 +372,12 @@ void k8s_dispatcher::handle_rc(const Json::Value& root, const msg_data& data)
 			os << "ADDED message received for existing replication controller [" << data.m_uid << "], updating only.";
 			g_logger.log(os.str(), sinsp_logger::SEV_INFO);
 		}
-		k8s_rc_s& rc = m_state.get_component<k8s_state_s::controllers, k8s_rc_s>(m_state.get_rcs(), data.m_name, data.m_uid, data.m_namespace);
+		k8s_rc_t& rc = m_state.get_component<k8s_controllers, k8s_rc_t>(m_state.get_rcs(), data.m_name, data.m_uid, data.m_namespace);
 		Json::Value object = root["object"];
 		if(!object.isNull())
 		{
-			Json::Value metadata = object["metadata"];
-			if(!metadata.isNull())
-			{
-				k8s_pair_list labels = k8s_component::extract_object(metadata, "labels");
-				if(labels.size() > 0)
-				{
-					rc.set_labels(std::move(labels));
-				}
-			}
-
-			Json::Value spec = object["spec"];
-			if(!spec.isNull())
-			{
-				k8s_pair_list selectors = k8s_component::extract_object(spec, "selector");
-				if(selectors.size() > 0)
-				{
-					rc.set_selectors(std::move(selectors));
-				}
-			}
+			handle_labels(rc, object["metadata"], "labels");
+			handle_selectors(rc, object["spec"], "selector");
 		}
 	}
 	else if(data.m_reason == COMPONENT_MODIFIED)
@@ -390,29 +389,12 @@ void k8s_dispatcher::handle_rc(const Json::Value& root, const msg_data& data)
 			g_logger.log(os.str(), sinsp_logger::SEV_ERROR);
 			return;
 		}
-		k8s_rc_s& rc = m_state.get_component<k8s_state_s::controllers, k8s_rc_s>(m_state.get_rcs(), data.m_name, data.m_uid, data.m_namespace);
+		k8s_rc_t& rc = m_state.get_component<k8s_controllers, k8s_rc_t>(m_state.get_rcs(), data.m_name, data.m_uid, data.m_namespace);
 		Json::Value object = root["object"];
 		if(!object.isNull())
 		{
-			Json::Value metadata = object["metadata"];
-			if(!metadata.isNull())
-			{
-				k8s_pair_list labels = k8s_component::extract_object(metadata, "labels");
-				if(labels.size() > 0)
-				{
-					rc.add_labels(std::move(labels));
-				}
-			}
-
-			Json::Value spec = object["spec"];
-			if(!spec.isNull())
-			{
-				k8s_pair_list selectors = k8s_component::extract_object(spec, "selector");
-				if(selectors.size() > 0)
-				{
-					rc.add_selectors(std::move(selectors));
-				}
-			}
+			handle_labels(rc, object["metadata"], "labels");
+			handle_selectors(rc, object["spec"], "selector");
 		}
 	}
 	else if(data.m_reason == COMPONENT_DELETED)
@@ -424,7 +406,7 @@ void k8s_dispatcher::handle_rc(const Json::Value& root, const msg_data& data)
 	}
 	else // COMPONENT_ERROR
 	{
-		g_logger.log("Bad CONTROLLER watch message.", sinsp_logger::SEV_ERROR);
+		log_error(root, "REPLICATION CONTROLLER");
 	}
 }
 
@@ -443,16 +425,8 @@ void k8s_dispatcher::handle_service(const Json::Value& root, const msg_data& dat
 				os << "ADDED message received for existing service [" << data.m_uid << "], updating only.";
 				g_logger.log(os.str(), sinsp_logger::SEV_INFO);
 			}
-			k8s_service_s& service = m_state.get_component<k8s_state_s::services, k8s_service_s>(m_state.get_services(), data.m_name, data.m_uid, data.m_namespace);
-			Json::Value metadata = object["metadata"];
-			if(!metadata.isNull())
-			{
-				k8s_pair_list entries = k8s_component::extract_object(metadata, "labels");
-				if(entries.size() > 0)
-				{
-					service.set_labels(std::move(entries));
-				}
-			}
+			k8s_service_t& service = m_state.get_component<k8s_services, k8s_service_t>(m_state.get_services(), data.m_name, data.m_uid, data.m_namespace);
+			handle_labels(service, object["metadata"], "labels");
 			k8s_component::extract_services_data(object, service, m_state.get_pods());
 		}
 	}
@@ -468,16 +442,8 @@ void k8s_dispatcher::handle_service(const Json::Value& root, const msg_data& dat
 				g_logger.log(os.str(), sinsp_logger::SEV_ERROR);
 				return;
 			}
-			k8s_service_s& service = m_state.get_component<k8s_state_s::services, k8s_service_s>(m_state.get_services(), data.m_name, data.m_uid, data.m_namespace);
-			Json::Value metadata = object["metadata"];
-			if(!metadata.isNull())
-			{
-				k8s_pair_list entries = k8s_component::extract_object(metadata, "labels");
-				if(entries.size() > 0)
-				{
-					service.add_labels(std::move(entries));
-				}
-			}
+			k8s_service_t& service = m_state.get_component<k8s_services, k8s_service_t>(m_state.get_services(), data.m_name, data.m_uid, data.m_namespace);
+			handle_labels(service, object["metadata"], "labels");
 			k8s_component::extract_services_data(object, service, m_state.get_pods());
 		}
 	}
@@ -490,7 +456,69 @@ void k8s_dispatcher::handle_service(const Json::Value& root, const msg_data& dat
 	}
 	else // COMPONENT_ERROR
 	{
-		g_logger.log("Bad SERVICE watch message.", sinsp_logger::SEV_ERROR);
+		log_error(root, "SERVICE");
+	}
+}
+
+void k8s_dispatcher::extract_data(const std::string& json, bool enqueue)
+{
+	Json::Value root;
+	Json::Reader reader;
+	if(reader.parse(json, root, false))
+	{
+		std::ostringstream os;
+		msg_data data = get_msg_data(root);
+		if(data.is_valid())
+		{
+			std::ostringstream os;
+			os << '[' << to_reason_desc(data.m_reason) << ',';
+			switch (m_type)
+			{
+				case k8s_component::K8S_NODES:
+					os << "NODE,";
+					handle_node(root, data);
+					break;
+				case k8s_component::K8S_NAMESPACES:
+					os << "NAMESPACE,";
+					handle_namespace(root, data);
+					break;
+				case k8s_component::K8S_PODS:
+					os << "POD,";
+					handle_pod(root, data);
+					break;
+				case k8s_component::K8S_REPLICATIONCONTROLLERS:
+					os << "REPLICATION_CONTROLLER,";
+					handle_rc(root, data);
+					break;
+				case k8s_component::K8S_SERVICES:
+					os << "SERVICE,";
+					handle_service(root, data);
+					break;
+				default:
+				{
+					std::ostringstream eos;
+					eos << "Unknown component: " << static_cast<int>(m_type);
+					throw sinsp_exception(os.str());
+				}
+			}
+			os << data.m_name << ',' << data.m_uid << ',' << data.m_namespace << ']';
+			g_logger.log(os.str(), sinsp_logger::SEV_INFO);
+			//g_logger.log(root.toStyledString(), sinsp_logger::SEV_DEBUG);
+			{
+				K8S_LOCK_GUARD_MUTEX;
+				m_state.update_cache(m_type);
+#ifdef HAS_CAPTURE
+				if(enqueue)
+				{
+					m_state.enqueue_capture_event(root);
+				}
+#endif
+			}
+		}
+	}
+	else
+	{
+		g_logger.log("Bad JSON message received.", sinsp_logger::SEV_ERROR);
 	}
 }
 
@@ -500,59 +528,7 @@ void k8s_dispatcher::dispatch()
 	{
 		if(is_ready(*it))
 		{
-			Json::Value root;
-			Json::Reader reader;
-			if(reader.parse(*it, root, false))
-			{
-				std::ostringstream os;
-				msg_data data = get_msg_data(root);
-				if(data.is_valid())
-				{
-					std::ostringstream os;
-					os << '[' << to_reason_desc(data.m_reason) << ',';
-					switch (m_type)
-					{
-						case k8s_component::K8S_NODES:
-							os << "NODE,";
-							handle_node(root, data);
-							break;
-						case k8s_component::K8S_NAMESPACES:
-							os << "NAMESPACE,";
-							handle_namespace(root, data);
-							break;
-						case k8s_component::K8S_PODS:
-							os << "POD,";
-							handle_pod(root, data);
-							break;
-						case k8s_component::K8S_REPLICATIONCONTROLLERS:
-							os << "REPLICATION_CONTROLLER,";
-							handle_rc(root, data);
-							break;
-						case k8s_component::K8S_SERVICES:
-							os << "SERVICE,";
-							handle_service(root, data);
-							break;
-						default:
-						{
-							std::ostringstream eos;
-							eos << "Unknown component: " << static_cast<int>(m_type);
-							throw sinsp_exception(os.str());
-						}
-					}
-					os << data.m_name << ',' << data.m_uid << ',' << data.m_namespace << ']';
-					g_logger.log(os.str(), sinsp_logger::SEV_INFO);
-					//g_logger.log(root.toStyledString(), sinsp_logger::SEV_DEBUG);
-					{
-						K8S_LOCK_GUARD_MUTEX;
-						m_state.update_cache(m_type);
-					}
-				}
-			}
-			else
-			{
-				// TODO: bad notification - discard or throw?
-				g_logger.log("Bad JSON message received.", sinsp_logger::SEV_ERROR);
-			}
+			extract_data(*it, true);
 			it = m_messages.erase(it);
 		}
 		else
