@@ -4,6 +4,8 @@
 
 #pragma once
 
+#ifdef HAS_CAPTURE
+
 #include "curl/curl.h"
 #include <iosfwd>
 #include <map>
@@ -19,7 +21,8 @@ public:
 		const std::string& host_and_port = "localhost:80",
 		const std::string& protocol = "http",
 		const std::string& credentials = "",
-		const std::string& api = "/api/v1");
+		const std::string& api = "/api/v1",
+		const std::string& cert = "");
 
 	~k8s_http();
 
@@ -36,9 +39,9 @@ public:
 private:
 	bool init();
 	void cleanup();
+	static size_t write_data(void *ptr, size_t size, size_t nmemb, void *cb);
 
 	int wait(curl_socket_t sockfd, int for_recv, long timeout_ms);
-	static size_t write_data(void *ptr, size_t size, size_t nmemb, void *cb);
 	static void check_error(CURLcode res);
 
 	CURL*         m_curl;
@@ -48,6 +51,7 @@ private:
 	std::string   m_api;
 	std::string   m_component;
 	std::string   m_credentials;
+	std::string   m_cert;
 	std::string   m_url;
 	curl_socket_t m_watch_socket;
 	bool          m_data_ready;
@@ -58,3 +62,4 @@ inline bool k8s_http::is_connected() const
 	return m_curl != 0;
 }
 
+#endif // HAS_CAPTURE
