@@ -2,6 +2,8 @@
 // k8s_net.cpp
 //
 
+#ifdef HAS_CAPTURE
+
 #include "k8s_net.h"
 #include "k8s_component.h"
 #include "k8s.h"
@@ -12,8 +14,12 @@
 #include <memory>
 
 
-k8s_net::k8s_net(k8s& kube, const std::string& uri, const std::string& api) : m_k8s(kube),
+k8s_net::k8s_net(k8s& kube, const std::string& uri,
+	const std::string& api,
+	const std::string& cert) : m_k8s(kube),
 		m_uri(uri + api),
+		m_api(api),
+		m_cert(cert),
 		m_stopped(true),
 		m_collector(kube.watch_in_thread())
 #ifndef K8S_DISABLE_THREAD
@@ -144,7 +150,7 @@ void k8s_net::get_all_data(const k8s_component::component_map::value_type& compo
 		{
 			os << ':' << port;
 		}
-		m_api_interfaces[component.first] = new k8s_http(m_k8s, component.second, os.str(), protocol, m_creds);
+		m_api_interfaces[component.first] = new k8s_http(m_k8s, component.second, os.str(), protocol, m_creds, m_api, m_cert);
 	}
 	
 	if(!m_api_interfaces[component.first]->get_all_data(out))
@@ -157,3 +163,4 @@ void k8s_net::get_all_data(const k8s_component::component_map::value_type& compo
 	}
 }
 
+#endif // HAS_CAPTURE
