@@ -5,6 +5,8 @@
 //
 #pragma once
 
+#ifdef HAS_CAPTURE
+
 #include "k8s_component.h"
 #include "k8s_event_data.h"
 #include "k8s_http.h"
@@ -22,7 +24,8 @@ class k8s_net
 {
 public:
 	k8s_net(k8s& kube, const std::string& uri = "http://localhost:80",
-		const std::string& api = "/api/v1/");
+		const std::string& api = "/api/v1/",
+		const std::string& cert = "");
 
 	~k8s_net();
 
@@ -56,6 +59,8 @@ private:
 	k8s&          m_k8s;
 	uri           m_uri;
 	std::string   m_creds;
+	std::string   m_api;
+	std::string   m_cert;
 	bool          m_stopped;
 	api_map_t     m_api_interfaces;
 	k8s_collector m_collector;
@@ -84,4 +89,5 @@ inline bool k8s_net::is_healthy() const
 		static_cast<int>(m_api_interfaces.size());
 }
 
+#endif // HAS_CAPTURE
 
