@@ -49,12 +49,23 @@ bool should_drop(sinsp_evt *evt);
 #endif
 
 extern sinsp_protodecoder_list g_decoderlist;
+extern sinsp_evttables g_infotables;
 
 sinsp_parser::sinsp_parser(sinsp *inspector) :
 	m_inspector(inspector),
 	m_tmp_evt(m_inspector),
 	m_fd_listener(NULL)
 {
+	m_k8s_metaevents_state.m_piscapevt = (scap_evt*)new char[SP_EVT_BUF_SIZE];
+	m_k8s_metaevents_state.m_piscapevt->type = PPME_K8S_E;
+
+	m_k8s_metaevents_state.m_metaevt.m_inspector = m_inspector;
+	m_k8s_metaevents_state.m_metaevt.m_info = &(g_infotables.m_event_info[PPME_SYSDIGEVENT_X]);
+	m_k8s_metaevents_state.m_metaevt.m_pevt = NULL;
+	m_k8s_metaevents_state.m_metaevt.m_cpuid = 0;
+	m_k8s_metaevents_state.m_metaevt.m_evtnum = 0;
+	m_k8s_metaevents_state.m_metaevt.m_pevt = m_k8s_metaevents_state.m_piscapevt;
+	m_k8s_metaevents_state.m_metaevt.m_fdinfo = NULL;
 }
 
 sinsp_parser::~sinsp_parser()
@@ -65,6 +76,7 @@ sinsp_parser::~sinsp_parser()
 	}
 
 	m_protodecoders.clear();
+	delete[] m_k8s_metaevents_state.m_piscapevt;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -309,6 +321,15 @@ void sinsp_parser::process_event(sinsp_evt *evt)
 	case PPME_CPU_HOTPLUG_E:
 		parse_cpu_hotplug_enter(evt);
 		break;
+	case PPME_K8S_E:
+		if(!m_inspector->is_live())
+		{
+			parse_k8s_evt(evt);
+		}
+		break;
+	case PPME_SYSCALL_CHROOT_X:
+		parse_chroot_exit(evt);
+		break;
 	default:
 		break;
 	}
@@ -938,6 +959,9 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 
 		// Copy the command arguments from the parent
 		tinfo.m_args = ptinfo->m_args;
+
+		// Copy the root from the parent
+		tinfo.m_root = ptinfo->m_root;
 	}
 	else
 	{
@@ -968,6 +992,7 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 			tinfo.m_comm = ptinfo->m_comm;
 			tinfo.m_exe = ptinfo->m_exe;
 			tinfo.m_args = ptinfo->m_args;
+			tinfo.m_root = ptinfo->m_root;
 		}
 		else
 		{
@@ -1217,7 +1242,7 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 		case PPME_SYSCALL_CLONE_20_X:
 			parinfo = evt->get_param(14);
 			tinfo.set_cgroups(parinfo->m_val, parinfo->m_len);
-			m_inspector->m_container_manager.resolve_container_from_cgroups(tinfo.m_cgroups, m_inspector->m_islive, &tinfo.m_container_id);
+			m_inspector->m_container_manager.resolve_container(&tinfo, m_inspector->m_islive);
 			break;
 	}
 
@@ -1387,7 +1412,7 @@ void sinsp_parser::parse_execve_exit(sinsp_evt *evt)
 		evt->m_tinfo->set_cgroups(parinfo->m_val, parinfo->m_len);
 		if(evt->m_tinfo->m_container_id.empty())
 		{
-			m_inspector->m_container_manager.resolve_container_from_cgroups(evt->m_tinfo->m_cgroups, m_inspector->m_islive, &evt->m_tinfo->m_container_id);
+			m_inspector->m_container_manager.resolve_container(evt->m_tinfo, m_inspector->m_islive);
 		}
 		break;
 	default:
@@ -1469,6 +1494,85 @@ void sinsp_parser::parse_openat_dir(sinsp_evt *evt, char* name, int64_t dirfd, O
 	}
 }
 
+void schedule_more_k8s_evts(sinsp* inspector, void* data)
+{
+#ifdef HAS_CAPTURE
+	ASSERT(data);
+	bool good_event = false;
+	k8s_metaevents_state* state = (k8s_metaevents_state*)data;
+
+	if(state->m_new_group == true)
+	{
+		state->m_new_group = false;
+		inspector->add_meta_event(&state->m_metaevt);
+		return;
+	}
+
+	k8s* k8s_client = inspector->get_k8s_client();
+	ASSERT(k8s_client);
+	if(!k8s_client->get_capture_events().size())
+	{
+		g_logger.log("K8S event scheduled but no events available."
+					"All pending K8S event request are cancelled.", sinsp_logger::SEV_ERROR);
+		state->m_new_group = false;
+		state->m_n_additional_k8s_events_to_add = 0;
+		inspector->remove_meta_event_callback();
+		return;
+	}
+	string payload = k8s_client->dequeue_capture_event();
+	std::size_t tot_len = sizeof(scap_evt) + sizeof(uint16_t) + payload.size() + 1;
+
+	if(tot_len <= SP_EVT_BUF_SIZE)
+	{
+		state->m_piscapevt->len = tot_len;
+		uint16_t* plen = (uint16_t*)((char *)state->m_piscapevt + sizeof(struct ppm_evt_hdr));
+		plen[0] = (uint16_t)payload.size() + 1;
+		uint8_t* edata = (uint8_t*)plen + sizeof(uint16_t);
+		memcpy(edata, payload.c_str(), plen[0]);
+		good_event = true;
+	}
+	else
+	{
+		g_logger.log("K8S event larger than available buffer, will not be recorded. "
+					"This may result in an inaccurate K8S event log.", sinsp_logger::SEV_ERROR);
+	}
+
+	state->m_n_additional_k8s_events_to_add--;
+	if(state->m_n_additional_k8s_events_to_add == 0)
+	{
+		inspector->remove_meta_event_callback();
+	}
+	else if(good_event)
+	{
+		inspector->add_meta_event(&state->m_metaevt);
+	}
+#endif // HAS_CAPTURE
+}
+
+void sinsp_parser::schedule_k8s_events(sinsp_evt *evt)
+{
+#ifdef HAS_CAPTURE
+	//
+	// schedule k8s events, if any available
+	//
+	k8s* k8s_client = 0;
+	if(m_inspector && (k8s_client = m_inspector->m_k8s_client))
+	{
+		int event_count = k8s_client->get_capture_events().size();
+		if(event_count)
+		{
+			m_k8s_metaevents_state.m_piscapevt->tid = evt->get_tid();
+			m_k8s_metaevents_state.m_piscapevt->ts = m_inspector->m_lastevent_ts;
+			m_k8s_metaevents_state.m_new_group = true;
+			m_k8s_metaevents_state.m_n_additional_k8s_events_to_add = event_count;
+			m_inspector->add_meta_event_callback(&schedule_more_k8s_evts, &m_k8s_metaevents_state);
+
+			schedule_more_k8s_evts(m_inspector, &m_k8s_metaevents_state);
+		}
+	}
+#endif // HAS_CAPTURE
+}
+
 void sinsp_parser::parse_open_openat_creat_exit(sinsp_evt *evt)
 {
 	sinsp_evt_param *parinfo;
@@ -1476,7 +1580,6 @@ void sinsp_parser::parse_open_openat_creat_exit(sinsp_evt *evt)
 	char *name;
 	uint32_t namelen;
 	uint32_t flags;
-	//  uint32_t mode;
 	sinsp_fdinfo_t fdi;
 	sinsp_evt *enter_evt = &m_tmp_evt;
 	string sdir;
@@ -3485,6 +3588,40 @@ uint8_t* sinsp_parser::reserve_event_buffer()
 	}
 }
 
+void sinsp_parser::parse_k8s_evt(sinsp_evt *evt)
+{
+	sinsp_evt_param *parinfo = evt->get_param(0);
+	ASSERT(parinfo);
+	ASSERT(parinfo->m_len > 0);
+	std::string json(parinfo->m_val, parinfo->m_len);
+	//g_logger.log(json, sinsp_logger::SEV_DEBUG);
+	ASSERT(m_inspector);
+	ASSERT(m_inspector->m_k8s_client);
+	m_inspector->m_k8s_client->simulate_watch_event(json);
+}
+
+void sinsp_parser::parse_chroot_exit(sinsp_evt *evt)
+{
+	auto parinfo = evt->get_param(0);
+	auto retval = *(int64_t *)parinfo->m_val;
+	if(retval == 0)
+	{
+		const char* resolved_path;
+		auto path = evt->get_param_as_str(1, &resolved_path);
+		if(resolved_path[0] == 0)
+		{
+			evt->m_tinfo->m_root = path;
+		}
+		else
+		{
+			evt->m_tinfo->m_root = resolved_path;
+		}
+		// Root change, let's detect if we are on a container
+		ASSERT(m_inspector);
+		m_inspector->m_container_manager.resolve_container(evt->m_tinfo, m_inspector->is_live());
+	}
+}
+
 void sinsp_parser::free_event_buffer(uint8_t *ptr)
 {
 	if(m_tmp_events_buffer.size() < m_inspector->m_thread_manager->m_threadtable.size())
