@@ -129,8 +129,15 @@ sinsp::sinsp() :
 	m_meinfo.m_pievt.m_fdinfo = NULL;
 	m_meinfo.m_n_procinfo_evts = 0;
 	m_meta_event_callback = NULL;
+	m_meta_event_callback_data = NULL;
 	m_k8s_client = NULL;
 	m_k8s_last_watch_time_ns = 0;
+
+	m_k8s_client = NULL;
+	m_k8s_api_server = NULL;
+	m_k8s_api_cert = NULL;
+
+	m_filter_proc_table_when_saving = false;
 }
 
 sinsp::~sinsp()
@@ -171,11 +178,9 @@ sinsp::~sinsp()
 		delete[] m_meinfo.m_piscapevt;
 	}
 
-	if(m_k8s_client)
-	{
-		delete m_k8s_client;
-		m_k8s_client = NULL;
-	}
+	delete m_k8s_client;
+	delete m_k8s_api_server;
+	delete m_k8s_api_cert;
 }
 
 void sinsp::add_protodecoders()
@@ -183,6 +188,16 @@ void sinsp::add_protodecoders()
 	m_parser->add_protodecoder("syslog");
 }
 
+void sinsp::filter_proc_table_when_saving(bool filter)
+{
+	m_filter_proc_table_when_saving = filter;
+
+	if(m_h != NULL)
+	{
+		scap_set_refresh_proc_table_when_saving(m_h, !filter);	
+	}
+}
+
 void sinsp::init()
 {
 	//
@@ -233,7 +248,56 @@ void sinsp::init()
 	m_n_proc_lookups = 0;
 	m_n_proc_lookups_duration_ns = 0;
 
+	//
+	// If we're reading from file, we try to pre-parse the container events before
+	// importing the thread table, so that thread table filtering will work with
+	// container filters
+	//
 	if(m_islive == false)
+	{
+		uint64_t off = scap_ftell(m_h);
+		scap_evt* pevent;
+		uint16_t pcpuid;
+		uint32_t ncnt = 0;
+
+		//
+		// Count how many container events we have
+		//
+		while(true)
+		{
+			int32_t res = scap_next(m_h, &pevent, &pcpuid);
+
+			if(res == SCAP_SUCCESS)
+			{
+				if(pevent->type != PPME_CONTAINER_E)
+				{
+					break;
+				}
+				else
+				{
+					ncnt++;
+					continue;
+				}
+			}
+			else
+			{
+				break;
+			}
+		}
+
+		//
+		// Rewind and consume the exact number of events
+		//
+		scap_fseek(m_h, off);
+
+		for(uint32_t j = 0; j < ncnt; j++)
+		{
+			sinsp_evt* tevt;
+			next(&tevt);
+		}
+	}
+
+	if(m_islive == false || m_filter_proc_table_when_saving == true)
 	{
 		import_thread_table();
 	}
@@ -304,8 +368,13 @@ void sinsp::open(uint32_t timeout_ms)
 	//
 	scap_open_args oargs;
 	oargs.fname = NULL;
-	oargs.proc_callback = ::on_new_entry_from_proc;
-	oargs.proc_callback_context = this;
+	oargs.proc_callback = NULL;
+	oargs.proc_callback_context = NULL;
+	if(!m_filter_proc_table_when_saving)
+	{
+		oargs.proc_callback = ::on_new_entry_from_proc;
+		oargs.proc_callback_context = this;
+	}
 	oargs.import_users = m_import_users;
 
 	m_h = scap_open(oargs, error);
@@ -315,12 +384,68 @@ void sinsp::open(uint32_t timeout_ms)
 		throw sinsp_exception(error);
 	}
 
+	scap_set_refresh_proc_table_when_saving(m_h, !m_filter_proc_table_when_saving);
+
 	init();
 }
 
+int64_t sinsp::get_file_size(const std::string& fname, char *error)
+{
+	static string err_str = "Could not determine capture file size: ";
+	std::string errdesc;
+#ifdef _WIN32
+	LARGE_INTEGER li = { 0 };
+	HANDLE fh = CreateFile(fname.c_str(), GENERIC_READ, FILE_SHARE_READ, NULL,
+		OPEN_EXISTING, FILE_ATTRIBUTE_READONLY, NULL);
+	if (fh != INVALID_HANDLE_VALUE)
+	{
+		if (0 != GetFileSizeEx(fh, &li))
+		{
+			CloseHandle(fh);
+			return li.QuadPart;
+		}
+		errdesc = get_error_desc(err_str);
+		CloseHandle(fh);
+	}
+#else
+	struct stat st;
+	if (0 == stat(fname.c_str(), &st))
+	{
+		return st.st_size;
+	}
+#endif
+	if(errdesc.empty()) errdesc = get_error_desc(err_str);
+	strncpy(error, errdesc.c_str(), errdesc.size() > SCAP_LASTERR_SIZE ? SCAP_LASTERR_SIZE : errdesc.size());
+	return -1;
+}
+
+std::string sinsp::get_error_desc(const std::string& msg)
+{
+#ifdef _WIN32
+	DWORD err_no = GetLastError(); // first, so error is not wiped out by intermediate calls
+	std::string errstr = msg;
+	DWORD flg = FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS;
+	LPTSTR msg_buf = 0;
+	if(FormatMessageA(flg, 0, err_no, 0, (LPTSTR)&msg_buf, 0, NULL))
+	if(msg_buf)
+	{
+		errstr.append(msg_buf, strlen(msg_buf));
+		LocalFree(msg_buf);
+	}
+#else
+	char* msg_buf = strerror(errno); // first, so error is not wiped out by intermediate calls
+	std::string errstr = msg;
+	if(msg_buf)
+	{
+		errstr.append(msg_buf, strlen(msg_buf));
+	}
+#endif
+	return errstr;
+}
+
 void sinsp::open(string filename)
 {
-	char error[SCAP_LASTERR_SIZE];
+	char error[SCAP_LASTERR_SIZE] = {0};
 
 	m_islive = false;
 
@@ -355,18 +480,11 @@ void sinsp::open(string filename)
 		throw sinsp_exception(error);
 	}
 
-	//
-	// gianluca: This might need to be replaced with
-	// a portable stat(), since I'm afraid that on S3
-	// (that we'll use in the backend) the seek will
-	// read the entire file anyway
-	//
-	FILE* fp = fopen(filename.c_str(), "rb");
-	if(fp)
+	m_filesize = get_file_size(filename, error);
+
+	if(m_filesize < 0)
 	{
-		fseek(fp, 0L, SEEK_END);
-		m_filesize = ftell(fp);
-		fclose(fp);
+		throw sinsp_exception(error);
 	}
 
 	init();
@@ -499,7 +617,8 @@ void sinsp::on_new_entry_from_proc(void* context,
 			}
 		}
 
-		sinsp_tinfo->add_fd(fdinfo);
+		sinsp_fdinfo_t sinsp_fdinfo;
+		sinsp_tinfo->add_fd_from_scap(fdinfo, &sinsp_fdinfo);
 	}
 }
 
@@ -593,6 +712,17 @@ void sinsp::add_meta_event_and_repeat(sinsp_evt *metaevt)
 	m_skipped_evt = &m_evt;
 }
 
+void sinsp::add_meta_event_callback(meta_event_callback cback, void* data)
+{
+	m_meta_event_callback = cback;
+	m_meta_event_callback_data = data;
+}
+
+void sinsp::remove_meta_event_callback()
+{
+	m_meta_event_callback = NULL;
+}
+
 void schedule_next_threadinfo_evt(sinsp* _this, void* data)
 {
 	sinsp_proc_metainfo* mei = (sinsp_proc_metainfo*)data;
@@ -655,7 +785,7 @@ int32_t sinsp::next(OUT sinsp_evt **puevt)
 
 		if(m_meta_event_callback != NULL)
 		{
-			m_meta_event_callback(this, &m_meinfo);
+			m_meta_event_callback(this, m_meta_event_callback_data);
 		}
 	}
 	else
@@ -752,7 +882,7 @@ int32_t sinsp::next(OUT sinsp_evt **puevt)
 
 						m_meinfo.m_piscapevt->ts = m_next_flush_time_ns - (ONE_SECOND_IN_NS + 1);
 						m_meinfo.m_next_evt = &m_evt;
-						m_meta_event_callback = &schedule_next_threadinfo_evt;
+						add_meta_event_callback(&schedule_next_threadinfo_evt, &m_meinfo);
 						schedule_next_threadinfo_evt(this, &m_meinfo);
 					}
 
@@ -967,52 +1097,10 @@ int32_t sinsp::next(OUT sinsp_evt **puevt)
 
 uint64_t sinsp::get_num_events()
 {
+	ASSERT(m_h);
 	return scap_event_get_num(m_h);
 }
 
-sinsp_threadinfo* sinsp::find_thread(int64_t tid, bool lookup_only)
-{
-	threadinfo_map_iterator_t it;
-
-	//
-	// Try looking up in our simple cache
-	//
-	if(m_thread_manager->m_last_tinfo && tid == m_thread_manager->m_last_tid)
-	{
-#ifdef GATHER_INTERNAL_STATS
-		m_thread_manager->m_cached_lookups->increment();
-#endif
-		m_thread_manager->m_last_tinfo->m_lastaccess_ts = m_lastevent_ts;
-		return m_thread_manager->m_last_tinfo;
-	}
-
-	//
-	// Caching failed, do a real lookup
-	//
-	it = m_thread_manager->m_threadtable.find(tid);
-	
-	if(it != m_thread_manager->m_threadtable.end())
-	{
-#ifdef GATHER_INTERNAL_STATS
-		m_thread_manager->m_non_cached_lookups->increment();
-#endif
-		if(!lookup_only)
-		{
-			m_thread_manager->m_last_tid = tid;
-			m_thread_manager->m_last_tinfo = &(it->second);
-			m_thread_manager->m_last_tinfo->m_lastaccess_ts = m_lastevent_ts;
-		}
-		return &(it->second);
-	}
-	else
-	{
-#ifdef GATHER_INTERNAL_STATS
-		m_thread_manager->m_failed_lookups->increment();
-#endif
-		return NULL;
-	}
-}
-
 sinsp_threadinfo* sinsp::find_thread_test(int64_t tid, bool lookup_only)
 {
 	return find_thread(tid, lookup_only);
@@ -1023,7 +1111,11 @@ sinsp_threadinfo* sinsp::get_thread(int64_t tid, bool query_os_if_not_found, boo
 	sinsp_threadinfo* sinsp_proc = find_thread(tid, lookup_only);
 
 	if(sinsp_proc == NULL && query_os_if_not_found &&
-	   (m_thread_manager->m_threadtable.size() < m_max_thread_table_size || tid == m_sysdig_pid))
+	   (m_thread_manager->m_threadtable.size() < m_max_thread_table_size
+#if defined(HAS_CAPTURE)
+		   || tid == m_sysdig_pid
+#endif
+		))
 	{
 		scap_threadinfo* scap_proc = NULL;
 		sinsp_threadinfo newti(this);
@@ -1419,7 +1511,7 @@ double sinsp::get_read_progress()
 
 	if(fpos == -1)
 	{
-		throw sinsp_exception(scap_getlasterr(m_h));		
+		throw sinsp_exception(scap_getlasterr(m_h));
 	}
 
 	return (double)fpos * 100 / m_filesize;
@@ -1430,14 +1522,23 @@ bool sinsp::remove_inactive_threads()
 	return m_thread_manager->remove_inactive_threads();
 }
 
-void sinsp::init_k8s_client(const string& api_server)
+void sinsp::init_k8s_client(string* api_server, string* ssl_cert)
 {
+	ASSERT(api_server);
 	m_k8s_api_server = api_server;
+	m_k8s_api_cert = ssl_cert;
 
 	if(m_k8s_client == NULL)
 	{
-		g_logger.log("Fetching initial k8s state");
-		m_k8s_client = new k8s(api_server, true);
+		g_logger.log("Fetching initial k8s state", sinsp_logger::SEV_INFO);
+		bool is_live = !m_k8s_api_server->empty();
+		m_k8s_client = new k8s(*m_k8s_api_server,
+			is_live ? true : false, // watch
+			false, // don't run watch in thread
+			is_live ? true : false, // capture
+			"/api/v1/",
+			m_k8s_api_cert ? *m_k8s_api_cert : string("")
+		);
 	}
 }
 
@@ -1451,8 +1552,9 @@ void sinsp::update_kubernetes_state()
 		if(m_k8s_client->is_alive())
 		{
 			uint64_t delta = sinsp_utils::get_current_time_ns();
-			
+
 			m_k8s_client->watch();
+			this->m_parser->schedule_k8s_events(&m_meta_evt);
 
 			delta = sinsp_utils::get_current_time_ns() - delta;
 
@@ -1463,7 +1565,7 @@ void sinsp::update_kubernetes_state()
 			g_logger.format(sinsp_logger::SEV_WARNING, "Kubernetes connection not active anymore, retrying");
 			delete m_k8s_client;
 			m_k8s_client = NULL;
-			init_k8s_client(m_k8s_api_server);
+			init_k8s_client(m_k8s_api_server, m_k8s_api_cert);
 		}
 	}
 }
