@@ -42,7 +42,7 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 
 #pragma once
 #ifdef _WIN32
-#pragma warning(disable: 4251 4200)
+#pragma warning(disable: 4251 4200 4221)
 #endif
 
 #ifdef _WIN32
@@ -542,7 +542,7 @@ public:
 	*/
 	inline bool is_live()
 	{
-		return m_islive;		
+		return m_islive;
 	}
 
 	/*!
@@ -648,7 +648,8 @@ public:
 	*/
 	double get_read_progress();
 
-	void init_k8s_client(const string& api_server);
+	void init_k8s_client(string* api_server, string* ssl_cert);
+	k8s* get_k8s_client() const { return m_k8s_client; }
 
 	//
 	// Misc internal stuff
@@ -675,6 +676,9 @@ public:
 	void import_ipv4_interface(const sinsp_ipv4_ifinfo& ifinfo);
 	void add_meta_event(sinsp_evt *metaevt);
 	void add_meta_event_and_repeat(sinsp_evt *metaevt);
+	void add_meta_event_callback(meta_event_callback cback, void* data);
+	void remove_meta_event_callback();
+	void filter_proc_table_when_saving(bool filter);
 
 	void refresh_ifaddr_list();
 
@@ -695,16 +699,60 @@ private:
 	void remove_thread(int64_t tid, bool force);
 	//
 	// Note: lookup_only should be used when the query for the thread is made
-	//       not as a consequence of an event for that thread arriving, but for
+	//       not as a consequence of an event for that thread arriving, but 
 	//       just for lookup reason. In that case, m_lastaccess_ts is not updated
 	//       and m_last_tinfo is not set.
 	//
-	inline sinsp_threadinfo* find_thread(int64_t tid, bool lookup_only);
+	inline sinsp_threadinfo* find_thread(int64_t tid, bool lookup_only)
+	{
+		threadinfo_map_iterator_t it;
+
+		//
+		// Try looking up in our simple cache
+		//
+		if(m_thread_manager->m_last_tinfo && tid == m_thread_manager->m_last_tid)
+		{
+	#ifdef GATHER_INTERNAL_STATS
+			m_thread_manager->m_cached_lookups->increment();
+	#endif
+			m_thread_manager->m_last_tinfo->m_lastaccess_ts = m_lastevent_ts;
+			return m_thread_manager->m_last_tinfo;
+		}
+
+		//
+		// Caching failed, do a real lookup
+		//
+		it = m_thread_manager->m_threadtable.find(tid);
+
+		if(it != m_thread_manager->m_threadtable.end())
+		{
+	#ifdef GATHER_INTERNAL_STATS
+			m_thread_manager->m_non_cached_lookups->increment();
+	#endif
+			if(!lookup_only)
+			{
+				m_thread_manager->m_last_tid = tid;
+				m_thread_manager->m_last_tinfo = &(it->second);
+				m_thread_manager->m_last_tinfo->m_lastaccess_ts = m_lastevent_ts;
+			}
+			return &(it->second);
+		}
+		else
+		{
+	#ifdef GATHER_INTERNAL_STATS
+			m_thread_manager->m_failed_lookups->increment();
+	#endif
+			return NULL;
+		}
+	}
 	// this is here for testing purposes only
 	sinsp_threadinfo* find_thread_test(int64_t tid, bool lookup_only);
 	bool remove_inactive_threads();
 	void update_kubernetes_state();
 
+	static int64_t get_file_size(const std::string& fname, char *error);
+	static std::string get_error_desc(const std::string& msg = "");
+
 	scap_t* m_h;
 	uint32_t m_nevts;
 	int64_t m_filesize;
@@ -726,6 +774,7 @@ private:
 	sinsp_parser* m_parser;
 	// the statistics analysis engine
 	scap_dumper_t* m_dumper;
+	bool m_filter_proc_table_when_saving;
 	const scap_machine_info* m_machine_info;
 	uint32_t m_num_cpus;
 	sinsp_thread_privatestate_manager m_thread_privatestate_manager;
@@ -739,7 +788,8 @@ private:
 	//
 	// Kubernetes stuff
 	//
-	string m_k8s_api_server;
+	string* m_k8s_api_server;
+	string* m_k8s_api_cert;
 	k8s* m_k8s_client;
 	uint64_t m_k8s_last_watch_time_ns;
 
@@ -826,6 +876,7 @@ private:
 	sinsp_evt* m_metaevt;
 	sinsp_evt* m_skipped_evt;
 	meta_event_callback m_meta_event_callback;
+	void* m_meta_event_callback_data;
 
 	//
 	// End of second housekeeping
@@ -868,6 +919,12 @@ private:
 // Used to eliminate mutex locking when running single-threaded
 //
 
+#ifndef HAS_CAPTURE
+#ifndef K8S_DISABLE_THREAD
+#define K8S_DISABLE_THREAD
+#endif
+#endif
+
 #ifndef K8S_DISABLE_THREAD
 #include <mutex>
 #define K8S_DECLARE_MUTEX mutable std::mutex m_mutex
