@@ -73,6 +73,12 @@ public:
 	*/
 	string get_cwd();
 
+	/*!
+	  \brief Return the value of the specified environment variable for the process
+	  containing this thread. Returns empty string if variable is not found.
+	*/
+	string get_env(const string& name) const;
+
 	/*!
 	  \brief Return true if this is a process' main thread.
 	*/
@@ -206,6 +212,7 @@ public:
 	uint64_t m_pfminor; ///< number of minor page faults since start.
 	int64_t m_vtid;  ///< The virtual id of this thread.
 	int64_t m_vpid; ///< The virtual id of the process containing this thread. In single thread threads, this is equal to vtid.
+	string m_root;
 
 	//
 	// State for multi-event processing
@@ -233,10 +240,11 @@ public:
 
 VISIBILITY_PRIVATE
 	void init();
-	void init(const scap_threadinfo* pi);
+	// return true if, based on the current inspector filter, this thread should be kept
+	void init(scap_threadinfo* pi);
 	void fix_sockets_coming_from_proc();
 	sinsp_fdinfo_t* add_fd(int64_t fd, sinsp_fdinfo_t *fdinfo);
-	void add_fd(scap_fdinfo *fdinfo);
+	void add_fd_from_scap(scap_fdinfo *fdinfo, OUT sinsp_fdinfo_t *res);
 	void remove_fd(int64_t fd);
 	inline sinsp_fdtable* get_fd_table()
 	{
