@@ -1,11 +1,10 @@
 --[[
 Copyright (C) 2014 Draios inc.
- 
+
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License version 2 as
 published by the Free Software Foundation.
 
-
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -16,21 +15,22 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 --]]
 
 -- Chisel description
-description = "List the running processes, with an output that is similar to the one of ps.";
+description = "List the running processes, with an output that is similar to the one of ps. Output is at a point in time; adjust this in the filter. It defaults to time of evt.num=0";
 short_description = "List (and optionally filter) the machine processes.";
 category = "System State";
-		   
+		
 -- Argument list
-args = 
+args =
 {
 	{
 		name = "filter",
-		description = "a sysdig-like filter expression that allows restricting the FD list. For example 'fd.name contains /etc' shows all the processes that have files open under /etc.", 
+		description = "A sysdig-like filter expression that allows restricting the FD list. For example 'fd.name contains /etc' shows all the processes that have files open under /etc.",
 		argtype = "filter",
 		optional = true
 	}
 }
 
+-- Argument initialization Callback
 function on_set_arg(name, val)
 	if name == "filter" then
 		filter = val
@@ -45,6 +45,7 @@ require "common"
 local dctable = {}
 local capturing = false
 local filter = nil
+local match = false
 
 -- Argument notification callback
 function on_set_arg(name, val)
@@ -69,14 +70,21 @@ end
 -- Event parsing callback
 function on_event()
 	sysdig.end_capture()
+	match = true
 	return false
 end
 
+-- Called by the engine at the end of the capture (Ctrl-C)
 function on_capture_end(ts_s, ts_ns, delta)
 	if not capturing then
 		return
 	end
 	
+	if match == false then
+		print("empty capture or no event matching the filter")
+		return
+	end
+	
 	local ttable = sysdig.get_thread_table(filter)
 
 	local sorted_ttable = pairs_top_by_val(ttable, 0, function(t,a,b) return a < b end)
