@@ -22,7 +22,7 @@ view_info =
 	description = "List every newly established network connection.",
 	tags = {"Default"},
 	view_type = "list",
-	applies_to = {"", "container.id", "proc.pid", "thread.tid", "proc.name", "fd.name", "fd.containername", "fd.sport", "fd.dport", "fd.port", "fd.lport", "fd.rport"},
+	applies_to = {"", "container.id", "proc.pid", "thread.tid", "proc.name", "fd.name", "fd.containername", "fd.sport", "fd.sproto", "fd.dport", "fd.port", "fd.lport", "fd.rport", "k8s.pod.id", "k8s.rc.id", "k8s.svc.id", "k8s.ns.id"},
 	filter = "evt.type=accept and evt.dir=< and evt.failed=false",
 	columns = 
 	{
