@@ -23,15 +23,15 @@ view_info =
 	tips = {"Want to restrict this visualization to a single process or container? Just drill down into them before applying it.", 
 		"Select a port and drill down with the 'Top Processes' view to see which processes are generating traffic on a port."},
 	view_type = "table",
-	applies_to = {"", "container.id", "proc.pid", "thread.tid", "proc.name", "evt.res"},
+	applies_to = {"", "container.id", "proc.pid", "thread.tid", "proc.name", "evt.res", "k8s.pod.id", "k8s.rc.id", "k8s.svc.id", "k8s.ns.id"},
 	filter = "fd.type=ipv4 and fd.name!=''",
 	use_defaults = true,
-	drilldown_target = "procs",
+	drilldown_target = "connections",
 	columns = 
 	{
 		{
 			name = "NA",
-			field = "fd.sproto",
+			field = "fd.sport",
 			is_key = true
 		},
 		{
