@@ -39,14 +39,16 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #else
 #include <unistd.h>
 #include <getopt.h>
+#include <term.h>
 #endif
 
 #include "cursescomponents.h"
 #include "cursestable.h"
 #include "cursesui.h"
 
-static bool g_terminate = false;
+#define MOUSE_CAPABLE_TERM "xterm-1002"
 
+static bool g_terminate = false;
 static void usage();
 
 //
@@ -82,6 +84,12 @@ static void usage()
 "                    specified as argument. E.g. \"http://admin:password@127.0.0.1:8080\".\n"
 "                    The API server can also be specified via the environment variable\n"
 "                    SYSDIG_K8S_API.\n"
+" -K, --k8s-api-cert=<file_name>\n"
+"                    Use the provided certificate file name to authenticate with the K8S API server.\n"
+"                    Filename must be a full absolute or relative (to the current directory) path\n"
+"                    to the certificate file.\n"
+"                    The certificate can also be specified via the environment variable.\n"
+"                    SYSDIG_K8S_API_CERT.\n"
 " -l, --list         List all the fields that can be used in views.\n"
 " --logfile=<file>\n"
 "                    Print program logs into the given file.\n"
@@ -223,7 +231,9 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 	uint64_t refresh_interval_ns = 2000000000;
 	bool list_flds = false;
 	bool m_raw_output = false;
-	string k8s_api;
+	string* k8s_api = 0;
+	string* k8s_api_cert = 0;
+	bool xt1002_available = false;
 
 	static struct option long_options[] =
 	{
@@ -231,6 +241,7 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 		{"exclude-users", no_argument, 0, 'E' },
 		{"help", no_argument, 0, 'h' },
 		{"k8s-api", required_argument, 0, 'k'},
+		{"k8s-api-cert", required_argument, 0, 'K' },
 		{"list", optional_argument, 0, 'l' },
 		{"numevents", required_argument, 0, 'n' },
 		{"print", required_argument, 0, 'p' },
@@ -258,7 +269,7 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 		// Parse the args
 		//
 		while((op = getopt_long(argc, argv,
-			"d:Ehk:lNn:p:r:s:v:", long_options, &long_index)) != -1)
+			"d:Ehk:K:lNn:p:r:s:v:", long_options, &long_index)) != -1)
 		{
 			switch(op)
 			{
@@ -292,7 +303,10 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 				delete inspector;
 				return sysdig_init_res(EXIT_SUCCESS);
 			case 'k':
-				k8s_api = optarg;
+				k8s_api = new string(optarg);
+				break;
+			case 'K':
+				k8s_api_cert = new string(optarg);
 				break;
 			case 'l':
 				list_flds = true;
@@ -327,6 +341,7 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 				break;
 			case 'r':
 				infiles.push_back(optarg);
+				k8s_api = new string();
 				break;
 			case 's':
 				snaplen = atoi(optarg);
@@ -417,17 +432,30 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 #ifndef NOCURSESUI
 		if(!m_raw_output)
 		{
+			//
+			// Check if xterm-1002 is available
+			//
+			xt1002_available =(tgetent(NULL, MOUSE_CAPABLE_TERM) != 0);
+
+			if(xt1002_available)
+			{
+				//
+				// Enable fine-grained mouse activity capture by setting xterm-1002
+				//
+				setenv("TERM", MOUSE_CAPABLE_TERM, 1);
+			}
+
 			(void) initscr();      // initialize the curses library
 			(void) nonl();         // tell curses not to do NL->CR/NL on output
 			intrflush(stdscr, false);
 			keypad(stdscr, true);
 			curs_set(0);
-			if (has_colors())
+			if(has_colors())
 			{
 			  start_color();
 			}
 			use_default_colors();
-			mousemask(ALL_MOUSE_EVENTS, NULL);
+			mousemask(ALL_MOUSE_EVENTS | REPORT_MOUSE_POSITION, NULL);
 			noecho();
 
 			timeout(0);
@@ -478,19 +506,6 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 		//
 		view_manager.set_selected_view(display_view);
 
-		if(!k8s_api.empty())
-		{
-			inspector->init_k8s_client(k8s_api);			
-		}
-		else
-		{
-			char* k8s_api_env = getenv("SYSDIG_K8S_API");
-			if(k8s_api_env != NULL)
-			{
-				inspector->init_k8s_client(k8s_api_env);
-			}
-		}
-
 		//
 		// Go through the input sources and apply the processing to all of them
 		//
@@ -504,7 +519,8 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 				(filter.size() != 0)? filter : "",
 				refresh_interval_ns,
 				print_containers,
-				m_raw_output);
+				m_raw_output,
+				xt1002_available);
 
 			ui.configure(&view_manager);
 			ui.start(false, false);
@@ -578,6 +594,45 @@ sysdig_init_res csysdig_init(int argc, char **argv)
 				inspector->set_snaplen(snaplen);
 			}
 
+			//
+			// run k8s, if required
+			//
+			if(k8s_api)
+			{
+				if(!k8s_api_cert)
+				{
+					if(char* k8s_cert_env = getenv("SYSDIG_K8S_API_CERT"))
+					{
+						k8s_api_cert = new string(k8s_cert_env);
+					}
+				}
+				inspector->init_k8s_client(k8s_api, k8s_api_cert);
+				k8s_api = 0;
+				k8s_api_cert = 0;
+			}
+			else if(char* k8s_api_env = getenv("SYSDIG_K8S_API"))
+			{
+				if(k8s_api_env != NULL)
+				{
+					if(!k8s_api_cert)
+					{
+						if(char* k8s_cert_env = getenv("SYSDIG_K8S_API_CERT"))
+						{
+							k8s_api_cert = new string(k8s_cert_env);
+						}
+					}
+					k8s_api = new string(k8s_api_env);
+					inspector->init_k8s_client(k8s_api, k8s_api_cert);
+				}
+				else
+				{
+					delete k8s_api;
+					delete k8s_api_cert;
+				}
+				k8s_api = 0;
+				k8s_api_cert = 0;
+			}
+
 			//
 			// Start the capture loop
 			//
@@ -636,6 +691,9 @@ int main(int argc, char **argv)
 {
 	sysdig_init_res res;
 
+	//
+	// Run csysdig
+	//
 	res = csysdig_init(argc, argv);
 
 #ifdef _WIN32
