@@ -106,7 +106,7 @@ static void usage()
 "                    parameter each.\n"
 "                    Used alongside -W flags creates a ring buffer of file containing\n"
 "                    num_events each.\n"
-" -F, --fatfile	     Enable fatfile mode\n"
+" -F, --fatfile      Enable fatfile mode\n"
 "                    when writing in fatfile mode, the output file will contain\n"
 "                    events that will be invisible when reading the file, but\n"
 "                    that are necessary to fully reconstruct the state.\n"
@@ -117,6 +117,12 @@ static void usage()
 "                    'hidden' so that they won't appear when reading the file.\n"
 "                    Be aware that using this flag might generate substantially\n"
 "                    bigger traces files.\n"
+" --filter-proclist  apply the filter to the process table\n"
+"                    a full dump of /proc is typically included in any trace file\n"
+"                    to make sure all the state required to decode events is in the\n"
+"                    file. This could cause the file to contain unwanted or sensitive\n"
+"                    information. Using this flag causes the command line filter to\n"
+"                    be applied to the /proc dump as well.\n"
 " -G <num_seconds>, --seconds=<num_seconds>\n"
 "                    Rotates the dump file specified with the -w option every\n"
 "                    num_seconds seconds. Savefiles will have the name specified\n"
@@ -132,14 +138,22 @@ static void usage()
 #endif
 " -j, --json         Emit output as json, data buffer encoding will depend from the\n"
 "                    print format selected.\n"
-" -k, --k8s-api      Enable Kubernetes support by connecting to the API server\n"
+" -k, --k8s-api <url>\n"
+"                    Enable Kubernetes support by connecting to the API server\n"
 "                    specified as argument. E.g. \"http://admin:password@127.0.0.1:8080\".\n"
 "                    The API server can also be specified via the environment variable\n"
 "                    SYSDIG_K8S_API.\n"
+" -K <file_name>, --k8s-api-cert=<file_name>\n"
+"                    Use the provided certificate file name to authenticate with the K8S API server.\n"
+"                    Filename must be a full absolute or relative (to the current directory) path\n"
+"                    to the certificate file.\n"
+"                    The certificate can also be specified via the environment variable\n"
+"                    SYSDIG_K8S_API_CERT.\n"
 " -L, --list-events  List the events that the engine supports\n"
 " -l, --list         List the fields that can be used for filtering and output\n"
 "                    formatting. Use -lv to get additional information for each\n"
 "                    field.\n"
+" -M <num_seconds>   Stop collecting after <num_seconds> reached.\n"
 " -N                 Don't convert port numbers to names.\n"
 " -n <num>, --numevents=<num>\n"
 "                    Stop capturing after <num> events\n"
@@ -470,6 +484,7 @@ void handle_end_of_file(bool print_progress, sinsp_evt_formatter* formatter = NU
 //
 captureinfo do_inspect(sinsp* inspector,
 	uint64_t cnt,
+	int duration_to_tot,
 	bool quiet,
 	bool json,
 	bool do_flush,
@@ -483,6 +498,7 @@ captureinfo do_inspect(sinsp* inspector,
 	sinsp_evt* ev;
 	string line;
 	double last_printed_progress_pct = 0;
+        int duration_start = 0;
 
 	if(json)
 	{
@@ -492,8 +508,18 @@ captureinfo do_inspect(sinsp* inspector,
 	//
 	// Loop through the events
 	//
+	duration_start = ((double)clock()) / CLOCKS_PER_SEC;
 	while(1)
 	{
+		if(duration_to_tot > 0)
+		{
+			int duration_tot = ((double)clock()) / CLOCKS_PER_SEC - duration_start;
+			if(duration_tot >= duration_to_tot)
+			{
+				handle_end_of_file(print_progress, formatter);
+				break;
+			}
+		}
 		if(retval.m_nevts == cnt || g_terminate)
 		{
 			//
@@ -659,6 +685,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 	sinsp_evt::param_fmt event_buffer_format = sinsp_evt::PF_NORMAL;
 	sinsp_filter* display_filter = NULL;
 	double duration = 1;
+	int duration_to_tot = 0;
 	captureinfo cinfo;
 	string output_format;
 	uint32_t snaplen = 0;
@@ -667,12 +694,14 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 	int cflag = 0;
 	bool jflag = false;
 	bool unbuf_flag = false;
+	bool filter_proclist_flag = false;
 	string cname;
 	vector<summary_table_entry>* summary_table = NULL;
-	string k8s_api;
+	string* k8s_api = 0;
+	string* k8s_api_cert = 0;
 
 	// These variables are for the cycle_writer engine
-	int duration_seconds = 0;	
+	int duration_seconds = 0;
 	int rollover_mb = 0;
 	int file_limit = 0;
 	unsigned long event_limit = 0L;
@@ -690,6 +719,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 		{"exclude-users", no_argument, 0, 'E' },
 		{"event-limit", required_argument, 0, 'e'},
 		{"fatfile", no_argument, 0, 'F'},
+		{"filter-proclist", no_argument, 0, 0 },
 		{"seconds", required_argument, 0, 'G' },
 		{"help", no_argument, 0, 'h' },
 #ifdef HAS_CHISELS
@@ -698,6 +728,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 		{"file-size", required_argument, 0, 'C' },
 		{"json", no_argument, 0, 'j' },
 		{"k8s-api", required_argument, 0, 'k'},
+		{"k8s-api-cert", required_argument, 0, 'K' },
 		{"list", no_argument, 0, 'l' },
 		{"list-events", no_argument, 0, 'L' },
 		{"numevents", required_argument, 0, 'n' },
@@ -738,7 +769,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
                                         "C:"
                                         "dDEe:F"
                                         "G:"
-                                        "hi:jk:lLNn:Pp:qr:Ss:t:v"
+                                        "hi:jk:K:lLM:Nn:Pp:qr:Ss:t:v"
                                         "W:"
                                         "w:xXz", long_options, &long_index)) != -1)
 		{
@@ -881,7 +912,10 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 				jflag = true;
 				break;
 			case 'k':
-				k8s_api = optarg;
+				k8s_api = new string(optarg);
+				break;
+			case 'K':
+				k8s_api_cert = new string(optarg);
 				break;
 			case 'h':
 				usage();
@@ -894,6 +928,15 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 				list_events(inspector);
 				delete inspector;
 				return sysdig_init_res(EXIT_SUCCESS);
+			case 'M':
+				duration_to_tot = atoi(optarg);
+				if(duration_to_tot <= 0)
+				{
+					throw sinsp_exception(string("invalid duration") + optarg);
+					res.m_res = EXIT_FAILURE;
+					goto exit;
+				}
+				break;
 			case 'N':
 				inspector->set_hostname_and_port_resolution_mode(false);
 				break;
@@ -962,6 +1005,7 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 				break;
 			case 'r':
 				infiles.push_back(optarg);
+				k8s_api = new string();
 				break;
 			case 'S':
 				summary_table = new vector<summary_table_entry>;
@@ -1058,6 +1102,11 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 			{
 				unbuf_flag = true;
 			}
+
+			if(string(long_options[long_index].name) == "filter-proclist")
+			{
+				filter_proclist_flag = true;
+			}
 		}
 
 		//
@@ -1167,16 +1216,27 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 			inspector->set_max_evt_output_len(80);
 		}
 
-		if(!k8s_api.empty())
+		//
+		// Determine if we need to filter when dumping to file
+		//
+		if(filter_proclist_flag)
 		{
-			inspector->init_k8s_client(k8s_api);			
-		}
-		else
-		{
-			char* k8s_api_env = getenv("SYSDIG_K8S_API");
-			if(k8s_api_env != NULL)
+			if(filter != "")
 			{
-				inspector->init_k8s_client(k8s_api_env);
+				if(infiles.size() == 0)
+				{
+					fprintf(stderr, "--filter-proclist not supported with live captures.\n");
+					res.m_res = EXIT_FAILURE;
+					goto exit;
+				}
+
+				inspector->filter_proc_table_when_saving(true);
+			}
+			else
+			{
+				fprintf(stderr, "you must specify a filter if you use --filter-proclist.\n");
+				res.m_res = EXIT_FAILURE;
+				goto exit;
 			}
 		}
 
@@ -1279,8 +1339,48 @@ sysdig_init_res sysdig_init(int argc, char **argv)
 			//
 			chisels_on_capture_start();
 
+			//
+			// run k8s, if required
+			//
+			if(k8s_api)
+			{
+				if(!k8s_api_cert)
+				{
+					if(char* k8s_cert_env = getenv("SYSDIG_K8S_API_CERT"))
+					{
+						k8s_api_cert = new string(k8s_cert_env);
+					}
+				}
+				inspector->init_k8s_client(k8s_api, k8s_api_cert);
+				k8s_api = 0;
+				k8s_api_cert = 0;
+			}
+			else if(char* k8s_api_env = getenv("SYSDIG_K8S_API"))
+			{
+				if(k8s_api_env != NULL)
+				{
+					if(!k8s_api_cert)
+					{
+						if(char* k8s_cert_env = getenv("SYSDIG_K8S_API_CERT"))
+						{
+							k8s_api_cert = new string(k8s_cert_env);
+						}
+					}
+					k8s_api = new string(k8s_api_env);
+					inspector->init_k8s_client(k8s_api, k8s_api_cert);
+				}
+				else
+				{
+					delete k8s_api;
+					delete k8s_api_cert;
+				}
+				k8s_api = 0;
+				k8s_api_cert = 0;
+			}
+
 			cinfo = do_inspect(inspector,
 				cnt,
+                                duration_to_tot,
 				quiet,
 				jflag,
 				unbuf_flag,
